using System;

namespace ECAL
{
	/// <summary>
	/// Summary description for Goal.
	/// </summary>
	public class Goal
	{
		/// <summary>
		/// Position of the goal on the ground
		/// </summary>
		public double X;
		public double Y;
		/// <summary>
		/// Radius of the goal as a circle
		/// </summary>
		public double Radius;
		/// <summary>
		/// Load of the goal
		/// </summary>
		public double Load;
		/// <summary>
		/// Environment which the goal is located in
		/// </summary>
		private Environment Environment;
		public double MoveSpeed = 0.008;

		public Goal(Environment env)
		{
			Environment = env;
		}

		/// <summary>
		/// This routine of goal is called frequently (or the processor of mobile robot runs it itself)
		/// </summary>
		/// <param name="timerInterval">Time in milliseconds between two calls of this routine</param>
		public void DecideAndDo(int timerInterval)
		{
			double RotatingRadius = Environment.Distance(
				Environment.GroundWidth/2.0,
				Environment.GroundHeight/2.0,
				this.X,
				this.Y);
			double PresentAngle = Math.Atan2(this.Y-Environment.GroundHeight/2.0, this.X-Environment.GroundWidth/2.0);
			double MovingVal = this.MoveSpeed * timerInterval;
			double DeltaTheta = MovingVal/RotatingRadius;

			double NewX = Environment.GroundWidth/2.0 + RotatingRadius * Math.Cos(PresentAngle+DeltaTheta);
			double NewY = Environment.GroundHeight/2.0 + RotatingRadius * Math.Sin(PresentAngle+DeltaTheta);
			MoveTo(NewX,NewY);
		}

		public void MoveTo(double newX, double newY)
		{
			if ((newX >= 0) && (newX <= Environment.GroundWidth))
				this.X = newX;
			if ((newY >= 0) && (newY <= Environment.GroundHeight))
				this.Y = newY;
		}
	}
}
