using System;
using System.Drawing;

namespace ECAL
{
	/// <summary>
	/// An agent used for image processing purposes
	/// </summary>
	public class ImageAgent
	{
		public int X = 0;
		public int Y = 0;

		private int NextX = 0;
		private int NextY = 0;
//		private double NextBrightness = 0.0;
		private double NextR = 0.0;
		private double NextG = 0.0;
		private double NextB = 0.0;

		public Bitmap Image;

		public ImageAgent(Bitmap img)
		{
			Image = img;
		}

		public void Decide()
		{
			Color PixColor;

			Color ThisPixColor = Image.GetPixel(X, Y);

			double ThisPixBrightness = ThisPixColor.GetBrightness();
			double RightBrightness = -1.0;
			double LeftBrightness = -1.0;
			double BottomBrightness = -1.0;
			double TopBrightness = -1.0;

			double WithRightPixBrDiff = -1;
			double WithLeftPixBrDiff = -1;
			double WithTopPixBrDiff = -1;
			double WithBottomPixBrDiff = -1;

			if (X < Image.Size.Width-1)
			{
				PixColor = Image.GetPixel(X+1, Y);
				RightBrightness = PixColor.GetBrightness();

				WithRightPixBrDiff = Math.Abs(RightBrightness - ThisPixBrightness);
			}

			if (X > 0)
			{
				PixColor = Image.GetPixel(X-1, Y);
				LeftBrightness = PixColor.GetBrightness();

				WithLeftPixBrDiff = Math.Abs(LeftBrightness - ThisPixBrightness);
			}
			
			if (Y < Image.Size.Height-1)
			{
				PixColor = Image.GetPixel(X, Y+1);
				BottomBrightness = PixColor.GetBrightness();

				WithBottomPixBrDiff = Math.Abs(BottomBrightness - ThisPixBrightness);
			}
			
			if (Y > 0)
			{
				PixColor = Image.GetPixel(X, Y-1);
				TopBrightness = PixColor.GetBrightness();

				WithTopPixBrDiff = Math.Abs(TopBrightness - ThisPixBrightness);
			}

			double MaxBrightness = 0.0;
			
			int NewX = X;
			int NewY = Y;

			double MaxAllowedBrightnessDiff = 0.3;
			double MaxBrightnessDiff = 0.0;
			
//			if ((RightBrightness > MaxBrightness) && (Math.Abs(RightBrightness-PixBrightness) < MaxAllowedBrightnessDiff))
//			{
//				NewX = X + 1;
//				NewY = Y;
//				MaxBrightness = RightBrightness;
//			}
//			
//			if ((LeftBrightness > MaxBrightness) && (Math.Abs(LeftBrightness-PixBrightness) < MaxAllowedBrightnessDiff))
//			{
//				NewX = X - 1;
//				NewY = Y;
//				MaxBrightness = LeftBrightness;
//			}
//			
//			if ((TopBrightness > MaxBrightness) && (Math.Abs(TopBrightness-PixBrightness) < MaxAllowedBrightnessDiff))
//			{
//				NewX = X;
//				NewY = Y - 1;
//				MaxBrightness = TopBrightness;
//			}
//			
//			if ((BottomBrightness > MaxBrightness) && (Math.Abs(BottomBrightness-PixBrightness) < MaxAllowedBrightnessDiff))
//			{
//				NewX = X;
//				NewY = Y + 1;
//				MaxBrightness = BottomBrightness;
//			}
			
			if (WithRightPixBrDiff < 0.3)
				if (RightBrightness >= 0.0)
					if ((WithRightPixBrDiff >= MaxBrightnessDiff) && (WithRightPixBrDiff < MaxAllowedBrightnessDiff))
					{
						NewX = X + 1;
						NewY = Y;
						MaxBrightnessDiff = WithRightPixBrDiff;// Math.Abs(RightBrightness - ThisPixBrightness);
					}
			
			if (WithLeftPixBrDiff < 0.3)
				if (LeftBrightness >= 0.0)
					if ((WithLeftPixBrDiff >= MaxBrightnessDiff) && (WithLeftPixBrDiff < MaxAllowedBrightnessDiff))
					{
						if (X > 0)
						{
							NewX = X - 1;
							NewY = Y;
							MaxBrightnessDiff = WithLeftPixBrDiff;// Math.Abs(LeftBrightness - ThisPixBrightness);
						}
					}

			if (WithTopPixBrDiff < 0.3)
				if (TopBrightness >= 0.0)
					if ((WithTopPixBrDiff >= MaxBrightnessDiff) && (WithTopPixBrDiff < MaxAllowedBrightnessDiff))
					{
						if (Y > 0)
						{
							NewX = X;
							NewY = Y - 1;
							MaxBrightnessDiff = WithTopPixBrDiff;// Math.Abs(TopBrightness - ThisPixBrightness);
						}
					}

			if (WithBottomPixBrDiff < 0.3)
				if (BottomBrightness >= 0.0)
					if ((WithBottomPixBrDiff >= MaxBrightnessDiff) && (WithBottomPixBrDiff < MaxAllowedBrightnessDiff))
					{
						NewX = X;
						NewY = Y + 1;
						MaxBrightnessDiff = WithBottomPixBrDiff;// Math.Abs(BottomBrightness - ThisPixBrightness);
					}
			
			if ((NewX == X) && (NewY == Y)) // agent has not moved
			{
				NewX = X + 1;
				NewY = Y + 1;
			}

			NextX = NewX;
			NextY = NewY;

			int NumOfPixelsForAverage = 0;

			double OtherCellsBrightness = 0.0;
			if (RightBrightness >= 0.0)
			{
				OtherCellsBrightness = OtherCellsBrightness + RightBrightness;
				NumOfPixelsForAverage++;
			}
			if (LeftBrightness >= 0.0)
			{
				OtherCellsBrightness = OtherCellsBrightness + LeftBrightness;
				NumOfPixelsForAverage++;
			}
			if (TopBrightness >= 0.0)
			{
				OtherCellsBrightness = OtherCellsBrightness + TopBrightness;
				NumOfPixelsForAverage++;
			}
			if (BottomBrightness >= 0.0)
			{
				OtherCellsBrightness = OtherCellsBrightness + BottomBrightness;
				NumOfPixelsForAverage++;
			}

			double AverageBrightness = ThisPixBrightness;

			if (Math.Abs(ThisPixBrightness - OtherCellsBrightness/4.0) < 1.0)
			{				
				double TotalBrighness =
					ThisPixBrightness +
					OtherCellsBrightness;

				AverageBrightness = TotalBrighness / (NumOfPixelsForAverage+1);
			}

//			NextBrightness = AverageBrightness;
//			NextBrightness = PixBrightness * 0.9;

			NextR = 255 * AverageBrightness;//0.9 * PixColor.R ;//* AverageBrightness;
			NextG = 255 * AverageBrightness;//0.9 * PixColor.G ;//* AverageBrightness;
			NextB = 255 * AverageBrightness;//0.9 * PixColor.B ;//* AverageBrightness;
		}

		public void Do()
		{			
			Color PixCol = Image.GetPixel(X, Y);
			PixCol = Color.FromArgb((int)(NextR), (int)(NextG), (int)(NextB));

			Image.SetPixel(X, Y, PixCol);

			X = NextX;
			Y = NextY;
		}

		public void FastDecideAndDo()
		{
		}
	}
}
