using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;

namespace ECAL
{
	/// <summary>
	/// Summary description for frmMain.
	/// </summary>
	public class frmMain : System.Windows.Forms.Form
	{
		/// <summary>
		/// Environment of the program that the robots, obstacles, goal and...
		/// are located in
		/// </summary>
		private Environment Environment;

		/// <summary>
		/// Height of the picture, to be calculated once and no need for more
		/// </summary>
		private int picHeight;
		/// <summary>
		/// Width of the picture, to be calculated once and no need for more
		/// </summary>
		private int picWidth;
		/// <summary>
		/// How many pixels on the picture area are equal to one meter
		/// </summary>
		double PicPixPerMeter = 67;
		private int MainTimerInterval;
		Pen DarkBluePen = new Pen(Color.FromArgb(0,0,180));
		SolidBrush DarkBlueBrush = new SolidBrush(Color.FromArgb(0,0,80));
		Pen GreyPen = new Pen(Color.FromArgb(70,70,70));
		Pen RedPen = new Pen(Color.FromArgb(255,0,0));
		Pen DarkRedPen = new Pen(Color.FromArgb(100,0,0));
		SolidBrush DarkRedBrush = new SolidBrush(Color.FromArgb(100,0,0));
		Pen GreenPen = new Pen(Color.FromArgb(0,100,0));
		SolidBrush GreenBrush = new SolidBrush(Color.FromArgb(0,100,0));
		Random RandomNum = new Random();
		private bool RightMouseIsDown = false;
		private bool LeftMouseIsDown = false;
		private double LeftMouseX;
		private double LeftMouseY;
		/// <summary>
		/// Specifies if the robots on the ground are shown or not
		/// This option is supplied because redrawing the robots for
		/// each time step takes time and for bulk simulations the robots
		/// are not shown for faster performances
		/// </summary>
		private bool CanShowOnPicture = true;
		/// <summary>
		/// Specifies if the total error of the robots is shown on the
		/// related text label or not
		/// </summary>
		private bool CanShowErrors = false;
		private bool CanShowCoverageRatio = true;
		private bool CanShowNumOfRobotMoves = true;
		private bool CanReport = false;
		/// <summary>
		/// Report generator of the program
		/// This instance is used for generating report
		/// </summary>
		private ReportGenerator RepGen;
		/// <summary>
		/// Specifies if the cell values and information are shown
		/// on the ground or not
		/// </summary>
		private bool ShowsEnvCells = false;
		/// <summary>
		/// Specifies if the grid lines in the ground are shown or not
		/// </summary>
		private bool ShowsGrids = false;
		/// <summary>
		/// Distance in meters between two grid lines
		/// </summary>
		private double GridsDistance = 1.0;
		/// <summary>
		/// Present working mode of robots
		/// This mode is applied to robots added at each time
		/// </summary>
		RobotWorkingModes RobotsWorkMode = RobotWorkingModes.CompleteCoverage;
		/// <summary>
		/// A pen with black color
		/// </summary>
		Pen BlackPen = new Pen(Color.Black);
		/// <summary>
		/// A pen with light blue color
		/// </summary>
		Pen LightBluePen = new Pen(Color.LightSkyBlue);
		Pen LightPinkPen = new Pen(Color.LightPink);
		Pen LightGreenPen = new Pen(Color.LightGreen);
		SolidBrush LightPinkBrush = new SolidBrush(Color.LightPink);
		/// <summary>
		/// if True, means that it is changing the show value (flag) for
		/// cells of environment and hence it does not consider messages
		/// for changing check boxes and...
		/// </summary>
		private bool IsChangingShowEnvCells = false;
		/// <summary>
		/// if True, means that it is changing the show value (flag) for
		/// grids of environment and hence it does not consider messages
		/// for changing check boxes and...
		/// </summary>
		private bool IsChangingShowEnvGrids = false;
		/// <summary>
		/// A bitmap to be used for test
		/// </summary>
		Bitmap TempBMP;
		Bitmap MainBMP;

		private System.Windows.Forms.Timer tmrMain;
		private System.Windows.Forms.PictureBox picGround;
		private System.Windows.Forms.Button btnAddObstacle;
		private System.Windows.Forms.Button btnAddGoal;
		private System.Windows.Forms.Button btnChangeObstaclesLoad;
		private System.Windows.Forms.TextBox txtNewObstaclesLoad;
		private System.Windows.Forms.Button btnAddRobot;
		private System.Windows.Forms.TextBox txtNumOfRobotsToAdd;
		private System.Windows.Forms.Button btnTest;
		private System.Windows.Forms.Button btnChangeRobotsSpeed;
		private System.Windows.Forms.TextBox txtRobotsSpeed;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.TextBox txtGoalSpeed;
		private System.Windows.Forms.Button btnChangeGoalSpeed;
		private System.Windows.Forms.GroupBox groupBox4;
		private System.Windows.Forms.CheckBox chkShowsOnPicture;
		private System.Windows.Forms.GroupBox groupBox5;
		private System.Windows.Forms.CheckBox chkShowErrors;
		private System.Windows.Forms.Label lblErrorsValue;
		private System.Windows.Forms.Button btnMultipleSteps;
		private System.Windows.Forms.TextBox txtMultipleSteps;
		private System.Windows.Forms.CheckBox chkRecordData;
		private System.Windows.Forms.Button btnSaveReport;
		private System.Windows.Forms.TextBox txtReportFileName;
		private System.Windows.Forms.CheckBox chkAutomaticWork;
		private System.Windows.Forms.RadioButton rdbCorrcetionNone;
		private System.Windows.Forms.RadioButton rdbCorrectionAverage;
		private System.Windows.Forms.RadioButton rdbCorrectionWeightedAverage;
		private System.Windows.Forms.GroupBox groupBox6;
		private System.Windows.Forms.RadioButton rdbCorrectionOWA;
		private System.Windows.Forms.Button btnCleanUpEntirely;
		private System.Windows.Forms.GroupBox groupBox7;
		private System.Windows.Forms.RadioButton rdbRobWorkModeFollowTarget;
		private System.Windows.Forms.RadioButton rdbRobWorkModeCompCoverage;
		private System.Windows.Forms.GroupBox groupBox8;
		private System.Windows.Forms.CheckBox chkShowEnvCells;
		private System.Windows.Forms.CheckBox chkShowGrids;
		private System.Windows.Forms.Button btnOpenEnvironment;
		private System.Windows.Forms.Button btnApplyRobotSettings;
		private System.Windows.Forms.TextBox txtSpringLength;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.TextBox txtRobotSeeDomain;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.TextBox txtRobotLoad;
		private System.Windows.Forms.Button btnClearCells;
		private System.Windows.Forms.Label lblCoverageRatio;
		private System.Windows.Forms.CheckBox chkShowCoverageRatio;
		private System.Windows.Forms.Label lblNumOfRobotMoves;
		private System.Windows.Forms.CheckBox chkShowNumOfRobotMoves;
		private System.Windows.Forms.Button btnMakeFirstRobotLeader;
		private System.Windows.Forms.TextBox txtTest;
		private System.Windows.Forms.Button btnMoveRobotsToOrigin;
		private System.Windows.Forms.MainMenu menuMain;
		private System.Windows.Forms.MenuItem menEnvironment;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.MenuItem menEnvironmentOpen;
		private System.Windows.Forms.MenuItem menRobot;
		private System.Windows.Forms.MenuItem menRobotAdd;
		private System.Windows.Forms.MenuItem menRobotMakefirstrobotleader;
		private System.Windows.Forms.MenuItem menRobotMoverobotstoorigin;
		private System.Windows.Forms.MenuItem menView;
		private System.Windows.Forms.MenuItem menViewGrids;
		private System.Windows.Forms.MenuItem menViewCells;
		private System.Windows.Forms.Button btnCreateCreatures;
		private System.Windows.Forms.Button btnCreaturesSingleStep;
		private System.Windows.Forms.Button btnTestImage;
		private System.Windows.Forms.Button btnShowImage;
		private System.Windows.Forms.TextBox txtImageNumber;
		private System.Windows.Forms.Button btnImageSingleStep;
		private System.ComponentModel.IContainer components;

		public frmMain()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			FirstInits();
		}

		/// <summary>
		/// First inits of the program which are done at the beginning of the program
		/// </summary>
		private void FirstInits()
		{
			MainBMP = new Bitmap("D:/Thesis/Complete Coverage Stigmergy/Code/ECAL/Pictures/image.jpg");
			TempBMP = new Bitmap(MainBMP);
			Environment = new Environment(RandomNum, TempBMP);

			PicPixPerMeter = picGround.Width / Environment.GroundWidth;
			picGround.Height = (int)(Environment.GroundHeight * PicPixPerMeter);

			picHeight = picGround.Height;
			picWidth = picGround.Width;

			RepGen = new ReportGenerator(Environment);

			MainTimerInterval = tmrMain.Interval;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(frmMain));
			this.picGround = new System.Windows.Forms.PictureBox();
			this.tmrMain = new System.Windows.Forms.Timer(this.components);
			this.btnAddObstacle = new System.Windows.Forms.Button();
			this.btnAddGoal = new System.Windows.Forms.Button();
			this.btnChangeObstaclesLoad = new System.Windows.Forms.Button();
			this.txtNewObstaclesLoad = new System.Windows.Forms.TextBox();
			this.btnAddRobot = new System.Windows.Forms.Button();
			this.txtNumOfRobotsToAdd = new System.Windows.Forms.TextBox();
			this.btnTest = new System.Windows.Forms.Button();
			this.btnChangeRobotsSpeed = new System.Windows.Forms.Button();
			this.txtRobotsSpeed = new System.Windows.Forms.TextBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.btnMoveRobotsToOrigin = new System.Windows.Forms.Button();
			this.label3 = new System.Windows.Forms.Label();
			this.txtRobotLoad = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.txtRobotSeeDomain = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.btnApplyRobotSettings = new System.Windows.Forms.Button();
			this.txtSpringLength = new System.Windows.Forms.TextBox();
			this.rdbCorrectionOWA = new System.Windows.Forms.RadioButton();
			this.rdbCorrectionWeightedAverage = new System.Windows.Forms.RadioButton();
			this.rdbCorrectionAverage = new System.Windows.Forms.RadioButton();
			this.rdbCorrcetionNone = new System.Windows.Forms.RadioButton();
			this.btnMakeFirstRobotLeader = new System.Windows.Forms.Button();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.txtGoalSpeed = new System.Windows.Forms.TextBox();
			this.btnChangeGoalSpeed = new System.Windows.Forms.Button();
			this.groupBox4 = new System.Windows.Forms.GroupBox();
			this.chkShowsOnPicture = new System.Windows.Forms.CheckBox();
			this.groupBox5 = new System.Windows.Forms.GroupBox();
			this.lblNumOfRobotMoves = new System.Windows.Forms.Label();
			this.chkShowNumOfRobotMoves = new System.Windows.Forms.CheckBox();
			this.lblCoverageRatio = new System.Windows.Forms.Label();
			this.chkShowCoverageRatio = new System.Windows.Forms.CheckBox();
			this.lblErrorsValue = new System.Windows.Forms.Label();
			this.chkShowErrors = new System.Windows.Forms.CheckBox();
			this.btnMultipleSteps = new System.Windows.Forms.Button();
			this.txtMultipleSteps = new System.Windows.Forms.TextBox();
			this.chkRecordData = new System.Windows.Forms.CheckBox();
			this.btnSaveReport = new System.Windows.Forms.Button();
			this.txtReportFileName = new System.Windows.Forms.TextBox();
			this.chkAutomaticWork = new System.Windows.Forms.CheckBox();
			this.groupBox6 = new System.Windows.Forms.GroupBox();
			this.btnCleanUpEntirely = new System.Windows.Forms.Button();
			this.groupBox7 = new System.Windows.Forms.GroupBox();
			this.rdbRobWorkModeCompCoverage = new System.Windows.Forms.RadioButton();
			this.rdbRobWorkModeFollowTarget = new System.Windows.Forms.RadioButton();
			this.groupBox8 = new System.Windows.Forms.GroupBox();
			this.btnClearCells = new System.Windows.Forms.Button();
			this.chkShowGrids = new System.Windows.Forms.CheckBox();
			this.chkShowEnvCells = new System.Windows.Forms.CheckBox();
			this.btnOpenEnvironment = new System.Windows.Forms.Button();
			this.txtTest = new System.Windows.Forms.TextBox();
			this.menuMain = new System.Windows.Forms.MainMenu();
			this.menEnvironment = new System.Windows.Forms.MenuItem();
			this.menEnvironmentOpen = new System.Windows.Forms.MenuItem();
			this.menRobot = new System.Windows.Forms.MenuItem();
			this.menRobotAdd = new System.Windows.Forms.MenuItem();
			this.menRobotMakefirstrobotleader = new System.Windows.Forms.MenuItem();
			this.menRobotMoverobotstoorigin = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			this.menView = new System.Windows.Forms.MenuItem();
			this.menViewGrids = new System.Windows.Forms.MenuItem();
			this.menViewCells = new System.Windows.Forms.MenuItem();
			this.btnCreateCreatures = new System.Windows.Forms.Button();
			this.btnCreaturesSingleStep = new System.Windows.Forms.Button();
			this.btnTestImage = new System.Windows.Forms.Button();
			this.btnShowImage = new System.Windows.Forms.Button();
			this.txtImageNumber = new System.Windows.Forms.TextBox();
			this.btnImageSingleStep = new System.Windows.Forms.Button();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox4.SuspendLayout();
			this.groupBox5.SuspendLayout();
			this.groupBox6.SuspendLayout();
			this.groupBox7.SuspendLayout();
			this.groupBox8.SuspendLayout();
			this.SuspendLayout();
			// 
			// picGround
			// 
			this.picGround.BackColor = System.Drawing.Color.White;
			this.picGround.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picGround.Location = new System.Drawing.Point(8, 8);
			this.picGround.Name = "picGround";
			this.picGround.Size = new System.Drawing.Size(608, 576);
			this.picGround.TabIndex = 1;
			this.picGround.TabStop = false;
			this.picGround.Paint += new System.Windows.Forms.PaintEventHandler(this.picGround_Paint);
			this.picGround.MouseUp += new System.Windows.Forms.MouseEventHandler(this.picGround_MouseUp);
			this.picGround.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picGround_MouseMove);
			this.picGround.MouseDown += new System.Windows.Forms.MouseEventHandler(this.picGround_MouseDown);
			// 
			// tmrMain
			// 
			this.tmrMain.Enabled = true;
			this.tmrMain.Interval = 10;
			this.tmrMain.Tick += new System.EventHandler(this.tmrMain_Tick);
			// 
			// btnAddObstacle
			// 
			this.btnAddObstacle.BackColor = System.Drawing.SystemColors.Control;
			this.btnAddObstacle.ForeColor = System.Drawing.Color.Black;
			this.btnAddObstacle.Location = new System.Drawing.Point(16, 56);
			this.btnAddObstacle.Name = "btnAddObstacle";
			this.btnAddObstacle.Size = new System.Drawing.Size(128, 23);
			this.btnAddObstacle.TabIndex = 8;
			this.btnAddObstacle.Text = "Add Obstacle";
			this.btnAddObstacle.Click += new System.EventHandler(this.btnAddObstacle_Click);
			// 
			// btnAddGoal
			// 
			this.btnAddGoal.BackColor = System.Drawing.SystemColors.Control;
			this.btnAddGoal.ForeColor = System.Drawing.Color.Black;
			this.btnAddGoal.Location = new System.Drawing.Point(16, 32);
			this.btnAddGoal.Name = "btnAddGoal";
			this.btnAddGoal.Size = new System.Drawing.Size(128, 23);
			this.btnAddGoal.TabIndex = 9;
			this.btnAddGoal.Text = "Add G&oal";
			this.btnAddGoal.Click += new System.EventHandler(this.btnAddGoal_Click);
			// 
			// btnChangeObstaclesLoad
			// 
			this.btnChangeObstaclesLoad.BackColor = System.Drawing.SystemColors.Control;
			this.btnChangeObstaclesLoad.ForeColor = System.Drawing.Color.Black;
			this.btnChangeObstaclesLoad.Location = new System.Drawing.Point(16, 24);
			this.btnChangeObstaclesLoad.Name = "btnChangeObstaclesLoad";
			this.btnChangeObstaclesLoad.Size = new System.Drawing.Size(128, 23);
			this.btnChangeObstaclesLoad.TabIndex = 11;
			this.btnChangeObstaclesLoad.Text = "Change obstacles load";
			this.btnChangeObstaclesLoad.Click += new System.EventHandler(this.btnChangeObstaclesLoad_Click);
			// 
			// txtNewObstaclesLoad
			// 
			this.txtNewObstaclesLoad.Location = new System.Drawing.Point(152, 24);
			this.txtNewObstaclesLoad.Name = "txtNewObstaclesLoad";
			this.txtNewObstaclesLoad.Size = new System.Drawing.Size(40, 20);
			this.txtNewObstaclesLoad.TabIndex = 12;
			this.txtNewObstaclesLoad.Text = "1";
			// 
			// btnAddRobot
			// 
			this.btnAddRobot.BackColor = System.Drawing.SystemColors.Control;
			this.btnAddRobot.ForeColor = System.Drawing.Color.Black;
			this.btnAddRobot.Location = new System.Drawing.Point(16, 24);
			this.btnAddRobot.Name = "btnAddRobot";
			this.btnAddRobot.Size = new System.Drawing.Size(128, 23);
			this.btnAddRobot.TabIndex = 13;
			this.btnAddRobot.Text = "Add Robot";
			this.btnAddRobot.Click += new System.EventHandler(this.AddRobot);
			// 
			// txtNumOfRobotsToAdd
			// 
			this.txtNumOfRobotsToAdd.Location = new System.Drawing.Point(152, 24);
			this.txtNumOfRobotsToAdd.Name = "txtNumOfRobotsToAdd";
			this.txtNumOfRobotsToAdd.Size = new System.Drawing.Size(40, 20);
			this.txtNumOfRobotsToAdd.TabIndex = 14;
			this.txtNumOfRobotsToAdd.Text = "10";
			// 
			// btnTest
			// 
			this.btnTest.BackColor = System.Drawing.SystemColors.Control;
			this.btnTest.Location = new System.Drawing.Point(872, 472);
			this.btnTest.Name = "btnTest";
			this.btnTest.Size = new System.Drawing.Size(88, 23);
			this.btnTest.TabIndex = 15;
			this.btnTest.Text = "Test";
			this.btnTest.Click += new System.EventHandler(this.btnTest_Click);
			// 
			// btnChangeRobotsSpeed
			// 
			this.btnChangeRobotsSpeed.BackColor = System.Drawing.SystemColors.Control;
			this.btnChangeRobotsSpeed.ForeColor = System.Drawing.Color.Black;
			this.btnChangeRobotsSpeed.Location = new System.Drawing.Point(16, 56);
			this.btnChangeRobotsSpeed.Name = "btnChangeRobotsSpeed";
			this.btnChangeRobotsSpeed.Size = new System.Drawing.Size(128, 23);
			this.btnChangeRobotsSpeed.TabIndex = 18;
			this.btnChangeRobotsSpeed.Text = "Change Robots speed";
			this.btnChangeRobotsSpeed.Click += new System.EventHandler(this.btnChangeRobotsSpeed_Click);
			// 
			// txtRobotsSpeed
			// 
			this.txtRobotsSpeed.Location = new System.Drawing.Point(152, 56);
			this.txtRobotsSpeed.Name = "txtRobotsSpeed";
			this.txtRobotsSpeed.Size = new System.Drawing.Size(40, 20);
			this.txtRobotsSpeed.TabIndex = 19;
			this.txtRobotsSpeed.Text = "1";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.btnMoveRobotsToOrigin);
			this.groupBox1.Controls.Add(this.label3);
			this.groupBox1.Controls.Add(this.txtRobotLoad);
			this.groupBox1.Controls.Add(this.label2);
			this.groupBox1.Controls.Add(this.txtRobotSeeDomain);
			this.groupBox1.Controls.Add(this.label1);
			this.groupBox1.Controls.Add(this.btnApplyRobotSettings);
			this.groupBox1.Controls.Add(this.txtSpringLength);
			this.groupBox1.Controls.Add(this.rdbCorrectionOWA);
			this.groupBox1.Controls.Add(this.rdbCorrectionWeightedAverage);
			this.groupBox1.Controls.Add(this.rdbCorrectionAverage);
			this.groupBox1.Controls.Add(this.rdbCorrcetionNone);
			this.groupBox1.Controls.Add(this.btnAddRobot);
			this.groupBox1.Controls.Add(this.txtNumOfRobotsToAdd);
			this.groupBox1.Controls.Add(this.txtRobotsSpeed);
			this.groupBox1.Controls.Add(this.btnChangeRobotsSpeed);
			this.groupBox1.Controls.Add(this.btnMakeFirstRobotLeader);
			this.groupBox1.ForeColor = System.Drawing.Color.White;
			this.groupBox1.Location = new System.Drawing.Point(624, 232);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(208, 312);
			this.groupBox1.TabIndex = 20;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Robot";
			// 
			// btnMoveRobotsToOrigin
			// 
			this.btnMoveRobotsToOrigin.BackColor = System.Drawing.SystemColors.Control;
			this.btnMoveRobotsToOrigin.ForeColor = System.Drawing.Color.Black;
			this.btnMoveRobotsToOrigin.Location = new System.Drawing.Point(16, 152);
			this.btnMoveRobotsToOrigin.Name = "btnMoveRobotsToOrigin";
			this.btnMoveRobotsToOrigin.Size = new System.Drawing.Size(128, 23);
			this.btnMoveRobotsToOrigin.TabIndex = 39;
			this.btnMoveRobotsToOrigin.Text = "Move Robots to Origin";
			this.btnMoveRobotsToOrigin.Click += new System.EventHandler(this.MoveRobotsToOrigin);
			// 
			// label3
			// 
			this.label3.AutoSize = true;
			this.label3.Location = new System.Drawing.Point(58, 240);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(29, 16);
			this.label3.TabIndex = 30;
			this.label3.Text = "Load";
			// 
			// txtRobotLoad
			// 
			this.txtRobotLoad.Location = new System.Drawing.Point(96, 240);
			this.txtRobotLoad.Name = "txtRobotLoad";
			this.txtRobotLoad.Size = new System.Drawing.Size(40, 20);
			this.txtRobotLoad.TabIndex = 29;
			this.txtRobotLoad.Text = "0.2";
			// 
			// label2
			// 
			this.label2.AutoSize = true;
			this.label2.Location = new System.Drawing.Point(29, 216);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(58, 16);
			this.label2.TabIndex = 28;
			this.label2.Text = "See radius";
			// 
			// txtRobotSeeDomain
			// 
			this.txtRobotSeeDomain.Location = new System.Drawing.Point(96, 216);
			this.txtRobotSeeDomain.Name = "txtRobotSeeDomain";
			this.txtRobotSeeDomain.Size = new System.Drawing.Size(40, 20);
			this.txtRobotSeeDomain.TabIndex = 27;
			this.txtRobotSeeDomain.Text = "5.0";
			// 
			// label1
			// 
			this.label1.AutoSize = true;
			this.label1.Location = new System.Drawing.Point(16, 192);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(71, 16);
			this.label1.TabIndex = 26;
			this.label1.Text = "Spring length";
			// 
			// btnApplyRobotSettings
			// 
			this.btnApplyRobotSettings.BackColor = System.Drawing.SystemColors.Control;
			this.btnApplyRobotSettings.ForeColor = System.Drawing.Color.Black;
			this.btnApplyRobotSettings.Location = new System.Drawing.Point(152, 240);
			this.btnApplyRobotSettings.Name = "btnApplyRobotSettings";
			this.btnApplyRobotSettings.Size = new System.Drawing.Size(48, 23);
			this.btnApplyRobotSettings.TabIndex = 24;
			this.btnApplyRobotSettings.Text = "Apply";
			this.btnApplyRobotSettings.Click += new System.EventHandler(this.btnApplyRobotSettings_Click);
			// 
			// txtSpringLength
			// 
			this.txtSpringLength.Location = new System.Drawing.Point(96, 192);
			this.txtSpringLength.Name = "txtSpringLength";
			this.txtSpringLength.Size = new System.Drawing.Size(40, 20);
			this.txtSpringLength.TabIndex = 25;
			this.txtSpringLength.Text = "3.0";
			// 
			// rdbCorrectionOWA
			// 
			this.rdbCorrectionOWA.Location = new System.Drawing.Point(144, 112);
			this.rdbCorrectionOWA.Name = "rdbCorrectionOWA";
			this.rdbCorrectionOWA.Size = new System.Drawing.Size(56, 24);
			this.rdbCorrectionOWA.TabIndex = 23;
			this.rdbCorrectionOWA.Text = "OWA";
			this.rdbCorrectionOWA.CheckedChanged += new System.EventHandler(this.RadioCorrectionMethodChanged);
			// 
			// rdbCorrectionWeightedAverage
			// 
			this.rdbCorrectionWeightedAverage.Location = new System.Drawing.Point(24, 112);
			this.rdbCorrectionWeightedAverage.Name = "rdbCorrectionWeightedAverage";
			this.rdbCorrectionWeightedAverage.Size = new System.Drawing.Size(120, 24);
			this.rdbCorrectionWeightedAverage.TabIndex = 22;
			this.rdbCorrectionWeightedAverage.Text = "Weighted Average";
			this.rdbCorrectionWeightedAverage.CheckedChanged += new System.EventHandler(this.RadioCorrectionMethodChanged);
			// 
			// rdbCorrectionAverage
			// 
			this.rdbCorrectionAverage.Location = new System.Drawing.Point(88, 88);
			this.rdbCorrectionAverage.Name = "rdbCorrectionAverage";
			this.rdbCorrectionAverage.TabIndex = 21;
			this.rdbCorrectionAverage.Text = "Average";
			this.rdbCorrectionAverage.CheckedChanged += new System.EventHandler(this.RadioCorrectionMethodChanged);
			// 
			// rdbCorrcetionNone
			// 
			this.rdbCorrcetionNone.Checked = true;
			this.rdbCorrcetionNone.Location = new System.Drawing.Point(24, 88);
			this.rdbCorrcetionNone.Name = "rdbCorrcetionNone";
			this.rdbCorrcetionNone.Size = new System.Drawing.Size(56, 24);
			this.rdbCorrcetionNone.TabIndex = 20;
			this.rdbCorrcetionNone.TabStop = true;
			this.rdbCorrcetionNone.Text = "None";
			this.rdbCorrcetionNone.CheckedChanged += new System.EventHandler(this.RadioCorrectionMethodChanged);
			// 
			// btnMakeFirstRobotLeader
			// 
			this.btnMakeFirstRobotLeader.BackColor = System.Drawing.SystemColors.Control;
			this.btnMakeFirstRobotLeader.ForeColor = System.Drawing.Color.Black;
			this.btnMakeFirstRobotLeader.Location = new System.Drawing.Point(16, 280);
			this.btnMakeFirstRobotLeader.Name = "btnMakeFirstRobotLeader";
			this.btnMakeFirstRobotLeader.Size = new System.Drawing.Size(128, 23);
			this.btnMakeFirstRobotLeader.TabIndex = 38;
			this.btnMakeFirstRobotLeader.Text = "Make first robot leader";
			this.btnMakeFirstRobotLeader.Click += new System.EventHandler(this.MakeFirstRobotLeader);
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.txtGoalSpeed);
			this.groupBox2.Controls.Add(this.btnChangeGoalSpeed);
			this.groupBox2.Controls.Add(this.btnAddGoal);
			this.groupBox2.ForeColor = System.Drawing.Color.White;
			this.groupBox2.Location = new System.Drawing.Point(624, 112);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(208, 112);
			this.groupBox2.TabIndex = 21;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Goal";
			// 
			// txtGoalSpeed
			// 
			this.txtGoalSpeed.Location = new System.Drawing.Point(152, 64);
			this.txtGoalSpeed.Name = "txtGoalSpeed";
			this.txtGoalSpeed.Size = new System.Drawing.Size(40, 20);
			this.txtGoalSpeed.TabIndex = 15;
			this.txtGoalSpeed.Text = "0.008";
			// 
			// btnChangeGoalSpeed
			// 
			this.btnChangeGoalSpeed.BackColor = System.Drawing.SystemColors.Control;
			this.btnChangeGoalSpeed.ForeColor = System.Drawing.Color.Black;
			this.btnChangeGoalSpeed.Location = new System.Drawing.Point(16, 64);
			this.btnChangeGoalSpeed.Name = "btnChangeGoalSpeed";
			this.btnChangeGoalSpeed.Size = new System.Drawing.Size(128, 23);
			this.btnChangeGoalSpeed.TabIndex = 10;
			this.btnChangeGoalSpeed.Text = "Change goal speed";
			this.btnChangeGoalSpeed.Click += new System.EventHandler(this.btnChangeGoalSpeed_Click);
			// 
			// groupBox4
			// 
			this.groupBox4.Controls.Add(this.btnChangeObstaclesLoad);
			this.groupBox4.Controls.Add(this.txtNewObstaclesLoad);
			this.groupBox4.Controls.Add(this.btnAddObstacle);
			this.groupBox4.ForeColor = System.Drawing.Color.White;
			this.groupBox4.Location = new System.Drawing.Point(624, 8);
			this.groupBox4.Name = "groupBox4";
			this.groupBox4.Size = new System.Drawing.Size(208, 96);
			this.groupBox4.TabIndex = 23;
			this.groupBox4.TabStop = false;
			this.groupBox4.Text = "Obstacle";
			// 
			// chkShowsOnPicture
			// 
			this.chkShowsOnPicture.Checked = true;
			this.chkShowsOnPicture.CheckState = System.Windows.Forms.CheckState.Checked;
			this.chkShowsOnPicture.ForeColor = System.Drawing.Color.White;
			this.chkShowsOnPicture.Location = new System.Drawing.Point(248, 24);
			this.chkShowsOnPicture.Name = "chkShowsOnPicture";
			this.chkShowsOnPicture.TabIndex = 24;
			this.chkShowsOnPicture.Text = "Show on picture";
			this.chkShowsOnPicture.CheckedChanged += new System.EventHandler(this.chkShowsOnPicture_CheckedChanged);
			// 
			// groupBox5
			// 
			this.groupBox5.Controls.Add(this.lblNumOfRobotMoves);
			this.groupBox5.Controls.Add(this.chkShowNumOfRobotMoves);
			this.groupBox5.Controls.Add(this.lblCoverageRatio);
			this.groupBox5.Controls.Add(this.chkShowCoverageRatio);
			this.groupBox5.Controls.Add(this.lblErrorsValue);
			this.groupBox5.Controls.Add(this.chkShowErrors);
			this.groupBox5.Controls.Add(this.chkShowsOnPicture);
			this.groupBox5.ForeColor = System.Drawing.Color.White;
			this.groupBox5.Location = new System.Drawing.Point(624, 552);
			this.groupBox5.Name = "groupBox5";
			this.groupBox5.Size = new System.Drawing.Size(392, 120);
			this.groupBox5.TabIndex = 25;
			this.groupBox5.TabStop = false;
			this.groupBox5.Text = "Show Settings";
			// 
			// lblNumOfRobotMoves
			// 
			this.lblNumOfRobotMoves.Location = new System.Drawing.Point(200, 88);
			this.lblNumOfRobotMoves.Name = "lblNumOfRobotMoves";
			this.lblNumOfRobotMoves.Size = new System.Drawing.Size(184, 24);
			this.lblNumOfRobotMoves.TabIndex = 30;
			this.lblNumOfRobotMoves.Text = "0";
			this.lblNumOfRobotMoves.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// chkShowNumOfRobotMoves
			// 
			this.chkShowNumOfRobotMoves.Checked = true;
			this.chkShowNumOfRobotMoves.CheckState = System.Windows.Forms.CheckState.Checked;
			this.chkShowNumOfRobotMoves.Location = new System.Drawing.Point(16, 88);
			this.chkShowNumOfRobotMoves.Name = "chkShowNumOfRobotMoves";
			this.chkShowNumOfRobotMoves.Size = new System.Drawing.Size(184, 24);
			this.chkShowNumOfRobotMoves.TabIndex = 29;
			this.chkShowNumOfRobotMoves.Text = "Show Number of Robot Moves";
			this.chkShowNumOfRobotMoves.Click += new System.EventHandler(this.chkShowNumOfRobotMoves_Click);
			// 
			// lblCoverageRatio
			// 
			this.lblCoverageRatio.Location = new System.Drawing.Point(160, 56);
			this.lblCoverageRatio.Name = "lblCoverageRatio";
			this.lblCoverageRatio.Size = new System.Drawing.Size(224, 24);
			this.lblCoverageRatio.TabIndex = 28;
			this.lblCoverageRatio.Text = "0.0";
			this.lblCoverageRatio.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// chkShowCoverageRatio
			// 
			this.chkShowCoverageRatio.Checked = true;
			this.chkShowCoverageRatio.CheckState = System.Windows.Forms.CheckState.Checked;
			this.chkShowCoverageRatio.Location = new System.Drawing.Point(16, 56);
			this.chkShowCoverageRatio.Name = "chkShowCoverageRatio";
			this.chkShowCoverageRatio.Size = new System.Drawing.Size(136, 24);
			this.chkShowCoverageRatio.TabIndex = 27;
			this.chkShowCoverageRatio.Text = "Show Coverage Ratio";
			this.chkShowCoverageRatio.CheckedChanged += new System.EventHandler(this.chkShowCoverageRatio_CheckedChanged);
			// 
			// lblErrorsValue
			// 
			this.lblErrorsValue.Location = new System.Drawing.Point(104, 24);
			this.lblErrorsValue.Name = "lblErrorsValue";
			this.lblErrorsValue.Size = new System.Drawing.Size(88, 24);
			this.lblErrorsValue.TabIndex = 26;
			this.lblErrorsValue.Text = "0.0";
			this.lblErrorsValue.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// chkShowErrors
			// 
			this.chkShowErrors.Location = new System.Drawing.Point(16, 24);
			this.chkShowErrors.Name = "chkShowErrors";
			this.chkShowErrors.Size = new System.Drawing.Size(88, 24);
			this.chkShowErrors.TabIndex = 25;
			this.chkShowErrors.Text = "Show Errors";
			this.chkShowErrors.CheckedChanged += new System.EventHandler(this.chkShowErrors_CheckedChanged);
			// 
			// btnMultipleSteps
			// 
			this.btnMultipleSteps.BackColor = System.Drawing.SystemColors.Control;
			this.btnMultipleSteps.ForeColor = System.Drawing.Color.Black;
			this.btnMultipleSteps.Location = new System.Drawing.Point(312, 24);
			this.btnMultipleSteps.Name = "btnMultipleSteps";
			this.btnMultipleSteps.Size = new System.Drawing.Size(96, 23);
			this.btnMultipleSteps.TabIndex = 26;
			this.btnMultipleSteps.Text = "Multiple Steps";
			this.btnMultipleSteps.Click += new System.EventHandler(this.btnMultipleSteps_Click);
			// 
			// txtMultipleSteps
			// 
			this.txtMultipleSteps.Location = new System.Drawing.Point(416, 24);
			this.txtMultipleSteps.Name = "txtMultipleSteps";
			this.txtMultipleSteps.Size = new System.Drawing.Size(40, 20);
			this.txtMultipleSteps.TabIndex = 27;
			this.txtMultipleSteps.Text = "";
			// 
			// chkRecordData
			// 
			this.chkRecordData.ForeColor = System.Drawing.Color.White;
			this.chkRecordData.Location = new System.Drawing.Point(16, 24);
			this.chkRecordData.Name = "chkRecordData";
			this.chkRecordData.Size = new System.Drawing.Size(88, 24);
			this.chkRecordData.TabIndex = 28;
			this.chkRecordData.Text = "Record Data";
			this.chkRecordData.CheckedChanged += new System.EventHandler(this.chkRecordData_CheckedChanged);
			// 
			// btnSaveReport
			// 
			this.btnSaveReport.BackColor = System.Drawing.SystemColors.Control;
			this.btnSaveReport.ForeColor = System.Drawing.SystemColors.ControlText;
			this.btnSaveReport.Location = new System.Drawing.Point(112, 24);
			this.btnSaveReport.Name = "btnSaveReport";
			this.btnSaveReport.TabIndex = 29;
			this.btnSaveReport.Text = "Save";
			this.btnSaveReport.Click += new System.EventHandler(this.btnSaveReport_Click);
			// 
			// txtReportFileName
			// 
			this.txtReportFileName.Location = new System.Drawing.Point(192, 24);
			this.txtReportFileName.Name = "txtReportFileName";
			this.txtReportFileName.TabIndex = 30;
			this.txtReportFileName.Text = "";
			// 
			// chkAutomaticWork
			// 
			this.chkAutomaticWork.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(0)), ((System.Byte)(0)), ((System.Byte)(64)));
			this.chkAutomaticWork.Checked = true;
			this.chkAutomaticWork.CheckState = System.Windows.Forms.CheckState.Checked;
			this.chkAutomaticWork.ForeColor = System.Drawing.Color.White;
			this.chkAutomaticWork.Location = new System.Drawing.Point(488, 24);
			this.chkAutomaticWork.Name = "chkAutomaticWork";
			this.chkAutomaticWork.TabIndex = 31;
			this.chkAutomaticWork.Text = "Automatic";
			this.chkAutomaticWork.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			this.chkAutomaticWork.CheckedChanged += new System.EventHandler(this.chkAutomaticWork_CheckedChanged);
			// 
			// groupBox6
			// 
			this.groupBox6.Controls.Add(this.btnSaveReport);
			this.groupBox6.Controls.Add(this.txtReportFileName);
			this.groupBox6.Controls.Add(this.btnMultipleSteps);
			this.groupBox6.Controls.Add(this.txtMultipleSteps);
			this.groupBox6.Controls.Add(this.chkAutomaticWork);
			this.groupBox6.Controls.Add(this.chkRecordData);
			this.groupBox6.ForeColor = System.Drawing.Color.White;
			this.groupBox6.Location = new System.Drawing.Point(8, 616);
			this.groupBox6.Name = "groupBox6";
			this.groupBox6.Size = new System.Drawing.Size(608, 56);
			this.groupBox6.TabIndex = 32;
			this.groupBox6.TabStop = false;
			this.groupBox6.Text = "Report";
			// 
			// btnCleanUpEntirely
			// 
			this.btnCleanUpEntirely.BackColor = System.Drawing.SystemColors.Control;
			this.btnCleanUpEntirely.ForeColor = System.Drawing.Color.Black;
			this.btnCleanUpEntirely.Location = new System.Drawing.Point(24, 80);
			this.btnCleanUpEntirely.Name = "btnCleanUpEntirely";
			this.btnCleanUpEntirely.Size = new System.Drawing.Size(128, 23);
			this.btnCleanUpEntirely.TabIndex = 33;
			this.btnCleanUpEntirely.Text = "Clean Up Entirely";
			this.btnCleanUpEntirely.Click += new System.EventHandler(this.btnCleanUpEntirely_Click);
			// 
			// groupBox7
			// 
			this.groupBox7.Controls.Add(this.rdbRobWorkModeCompCoverage);
			this.groupBox7.Controls.Add(this.rdbRobWorkModeFollowTarget);
			this.groupBox7.ForeColor = System.Drawing.Color.White;
			this.groupBox7.Location = new System.Drawing.Point(840, 8);
			this.groupBox7.Name = "groupBox7";
			this.groupBox7.Size = new System.Drawing.Size(176, 96);
			this.groupBox7.TabIndex = 34;
			this.groupBox7.TabStop = false;
			this.groupBox7.Text = "Robots Working Mode";
			// 
			// rdbRobWorkModeCompCoverage
			// 
			this.rdbRobWorkModeCompCoverage.Checked = true;
			this.rdbRobWorkModeCompCoverage.Location = new System.Drawing.Point(16, 56);
			this.rdbRobWorkModeCompCoverage.Name = "rdbRobWorkModeCompCoverage";
			this.rdbRobWorkModeCompCoverage.Size = new System.Drawing.Size(128, 24);
			this.rdbRobWorkModeCompCoverage.TabIndex = 1;
			this.rdbRobWorkModeCompCoverage.TabStop = true;
			this.rdbRobWorkModeCompCoverage.Text = "Complete Coverage";
			this.rdbRobWorkModeCompCoverage.CheckedChanged += new System.EventHandler(this.RadoiRobsWorkModeChanged);
			// 
			// rdbRobWorkModeFollowTarget
			// 
			this.rdbRobWorkModeFollowTarget.Location = new System.Drawing.Point(16, 24);
			this.rdbRobWorkModeFollowTarget.Name = "rdbRobWorkModeFollowTarget";
			this.rdbRobWorkModeFollowTarget.Size = new System.Drawing.Size(128, 24);
			this.rdbRobWorkModeFollowTarget.TabIndex = 0;
			this.rdbRobWorkModeFollowTarget.Text = "Following Target";
			this.rdbRobWorkModeFollowTarget.CheckedChanged += new System.EventHandler(this.RadoiRobsWorkModeChanged);
			// 
			// groupBox8
			// 
			this.groupBox8.Controls.Add(this.btnClearCells);
			this.groupBox8.Controls.Add(this.chkShowGrids);
			this.groupBox8.Controls.Add(this.chkShowEnvCells);
			this.groupBox8.Controls.Add(this.btnOpenEnvironment);
			this.groupBox8.Controls.Add(this.btnCleanUpEntirely);
			this.groupBox8.ForeColor = System.Drawing.Color.White;
			this.groupBox8.Location = new System.Drawing.Point(840, 112);
			this.groupBox8.Name = "groupBox8";
			this.groupBox8.Size = new System.Drawing.Size(176, 192);
			this.groupBox8.TabIndex = 35;
			this.groupBox8.TabStop = false;
			this.groupBox8.Text = "Environment";
			// 
			// btnClearCells
			// 
			this.btnClearCells.BackColor = System.Drawing.SystemColors.Control;
			this.btnClearCells.ForeColor = System.Drawing.Color.Black;
			this.btnClearCells.Location = new System.Drawing.Point(24, 48);
			this.btnClearCells.Name = "btnClearCells";
			this.btnClearCells.Size = new System.Drawing.Size(128, 23);
			this.btnClearCells.TabIndex = 28;
			this.btnClearCells.Text = "Clear Cells";
			this.btnClearCells.Click += new System.EventHandler(this.btnClearCells_Click);
			// 
			// chkShowGrids
			// 
			this.chkShowGrids.Location = new System.Drawing.Point(8, 120);
			this.chkShowGrids.Name = "chkShowGrids";
			this.chkShowGrids.Size = new System.Drawing.Size(152, 24);
			this.chkShowGrids.TabIndex = 27;
			this.chkShowGrids.Text = "Show Grids";
			this.chkShowGrids.CheckedChanged += new System.EventHandler(this.ShowEnvGridsCheckedChanged);
			// 
			// chkShowEnvCells
			// 
			this.chkShowEnvCells.Location = new System.Drawing.Point(8, 16);
			this.chkShowEnvCells.Name = "chkShowEnvCells";
			this.chkShowEnvCells.Size = new System.Drawing.Size(160, 24);
			this.chkShowEnvCells.TabIndex = 26;
			this.chkShowEnvCells.Text = "Show Environment Cells";
			this.chkShowEnvCells.CheckedChanged += new System.EventHandler(this.ShowEnvCellsCheckedChanged);
			// 
			// btnOpenEnvironment
			// 
			this.btnOpenEnvironment.BackColor = System.Drawing.SystemColors.Control;
			this.btnOpenEnvironment.ForeColor = System.Drawing.Color.Black;
			this.btnOpenEnvironment.Location = new System.Drawing.Point(24, 152);
			this.btnOpenEnvironment.Name = "btnOpenEnvironment";
			this.btnOpenEnvironment.Size = new System.Drawing.Size(128, 23);
			this.btnOpenEnvironment.TabIndex = 36;
			this.btnOpenEnvironment.Text = "Open Environment";
			this.btnOpenEnvironment.Click += new System.EventHandler(this.OpenEnvironment);
			// 
			// txtTest
			// 
			this.txtTest.Location = new System.Drawing.Point(968, 472);
			this.txtTest.Name = "txtTest";
			this.txtTest.Size = new System.Drawing.Size(40, 20);
			this.txtTest.TabIndex = 36;
			this.txtTest.Text = "3.0";
			// 
			// menuMain
			// 
			this.menuMain.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menEnvironment,
																					 this.menRobot,
																					 this.menuItem3,
																					 this.menuItem4,
																					 this.menView});
			// 
			// menEnvironment
			// 
			this.menEnvironment.Index = 0;
			this.menEnvironment.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						   this.menEnvironmentOpen});
			this.menEnvironment.Text = "&Environment";
			// 
			// menEnvironmentOpen
			// 
			this.menEnvironmentOpen.Index = 0;
			this.menEnvironmentOpen.Shortcut = System.Windows.Forms.Shortcut.F3;
			this.menEnvironmentOpen.Text = "Open...";
			this.menEnvironmentOpen.Click += new System.EventHandler(this.OpenEnvironment);
			// 
			// menRobot
			// 
			this.menRobot.Index = 1;
			this.menRobot.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.menRobotAdd,
																					 this.menRobotMakefirstrobotleader,
																					 this.menRobotMoverobotstoorigin});
			this.menRobot.Text = "&Robot";
			// 
			// menRobotAdd
			// 
			this.menRobotAdd.Index = 0;
			this.menRobotAdd.Text = "&Add";
			this.menRobotAdd.Click += new System.EventHandler(this.AddRobot);
			// 
			// menRobotMakefirstrobotleader
			// 
			this.menRobotMakefirstrobotleader.Index = 1;
			this.menRobotMakefirstrobotleader.Text = "Make first robot &Leader";
			this.menRobotMakefirstrobotleader.Click += new System.EventHandler(this.MakeFirstRobotLeader);
			// 
			// menRobotMoverobotstoorigin
			// 
			this.menRobotMoverobotstoorigin.Index = 2;
			this.menRobotMoverobotstoorigin.Text = "Move robots to &origin";
			this.menRobotMoverobotstoorigin.Click += new System.EventHandler(this.MoveRobotsToOrigin);
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 2;
			this.menuItem3.Text = "Wall";
			// 
			// menuItem4
			// 
			this.menuItem4.Index = 3;
			this.menuItem4.Text = "Goal";
			// 
			// menView
			// 
			this.menView.Index = 4;
			this.menView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.menViewGrids,
																					this.menViewCells});
			this.menView.Text = "&View";
			// 
			// menViewGrids
			// 
			this.menViewGrids.Index = 0;
			this.menViewGrids.Text = "&Grids";
			this.menViewGrids.Click += new System.EventHandler(this.ShowEnvGridsCheckedChanged);
			// 
			// menViewCells
			// 
			this.menViewCells.Index = 1;
			this.menViewCells.Text = "&Cells";
			this.menViewCells.Click += new System.EventHandler(this.ShowEnvCellsCheckedChanged);
			// 
			// btnCreateCreatures
			// 
			this.btnCreateCreatures.BackColor = System.Drawing.SystemColors.Control;
			this.btnCreateCreatures.Location = new System.Drawing.Point(872, 312);
			this.btnCreateCreatures.Name = "btnCreateCreatures";
			this.btnCreateCreatures.Size = new System.Drawing.Size(136, 23);
			this.btnCreateCreatures.TabIndex = 37;
			this.btnCreateCreatures.Text = "Create Creatures";
			this.btnCreateCreatures.Click += new System.EventHandler(this.btnCreateCreatures_Click);
			// 
			// btnCreaturesSingleStep
			// 
			this.btnCreaturesSingleStep.BackColor = System.Drawing.SystemColors.Control;
			this.btnCreaturesSingleStep.Location = new System.Drawing.Point(872, 344);
			this.btnCreaturesSingleStep.Name = "btnCreaturesSingleStep";
			this.btnCreaturesSingleStep.Size = new System.Drawing.Size(136, 23);
			this.btnCreaturesSingleStep.TabIndex = 38;
			this.btnCreaturesSingleStep.Text = "Creatures Single Step";
			this.btnCreaturesSingleStep.Click += new System.EventHandler(this.btnCreaturesSingleStep_Click);
			// 
			// btnTestImage
			// 
			this.btnTestImage.BackColor = System.Drawing.SystemColors.Control;
			this.btnTestImage.Location = new System.Drawing.Point(872, 424);
			this.btnTestImage.Name = "btnTestImage";
			this.btnTestImage.Size = new System.Drawing.Size(136, 23);
			this.btnTestImage.TabIndex = 40;
			this.btnTestImage.Text = "Test Image";
			this.btnTestImage.Click += new System.EventHandler(this.btnTestImage_Click);
			// 
			// btnShowImage
			// 
			this.btnShowImage.BackColor = System.Drawing.SystemColors.Control;
			this.btnShowImage.Location = new System.Drawing.Point(872, 392);
			this.btnShowImage.Name = "btnShowImage";
			this.btnShowImage.Size = new System.Drawing.Size(136, 23);
			this.btnShowImage.TabIndex = 39;
			this.btnShowImage.Text = "Show Image";
			this.btnShowImage.Click += new System.EventHandler(this.btnShowImage_Click);
			// 
			// txtImageNumber
			// 
			this.txtImageNumber.Location = new System.Drawing.Point(968, 448);
			this.txtImageNumber.Name = "txtImageNumber";
			this.txtImageNumber.Size = new System.Drawing.Size(40, 20);
			this.txtImageNumber.TabIndex = 41;
			this.txtImageNumber.Text = "1.0";
			// 
			// btnImageSingleStep
			// 
			this.btnImageSingleStep.BackColor = System.Drawing.SystemColors.Control;
			this.btnImageSingleStep.Location = new System.Drawing.Point(872, 504);
			this.btnImageSingleStep.Name = "btnImageSingleStep";
			this.btnImageSingleStep.Size = new System.Drawing.Size(136, 23);
			this.btnImageSingleStep.TabIndex = 42;
			this.btnImageSingleStep.Text = "Image Single Step";
			this.btnImageSingleStep.Click += new System.EventHandler(this.btnImageSingleStep_Click);
			// 
			// frmMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(0)), ((System.Byte)(0)), ((System.Byte)(64)));
			this.ClientSize = new System.Drawing.Size(1028, 678);
			this.Controls.Add(this.btnImageSingleStep);
			this.Controls.Add(this.txtImageNumber);
			this.Controls.Add(this.btnTestImage);
			this.Controls.Add(this.btnShowImage);
			this.Controls.Add(this.btnCreaturesSingleStep);
			this.Controls.Add(this.btnCreateCreatures);
			this.Controls.Add(this.txtTest);
			this.Controls.Add(this.groupBox8);
			this.Controls.Add(this.groupBox7);
			this.Controls.Add(this.groupBox6);
			this.Controls.Add(this.groupBox5);
			this.Controls.Add(this.groupBox4);
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.btnTest);
			this.Controls.Add(this.picGround);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.menuMain;
			this.Name = "frmMain";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Mobile Robots Simulator";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox4.ResumeLayout(false);
			this.groupBox5.ResumeLayout(false);
			this.groupBox6.ResumeLayout(false);
			this.groupBox7.ResumeLayout(false);
			this.groupBox8.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}

		/// <summary>
		/// Draws cells of the environment
		/// </summary>
		/// <param name="e"></param>
		private void DrawEnvCells(System.Windows.Forms.PaintEventArgs e)
		{
			/// Drawing the vertical lines
			for(double Counter = 0 ; Counter < Environment.GroundWidth ; Counter = Counter + Environment.CellsWidth)
			{
				int X = (int)(PicPixPerMeter * Counter);
				e.Graphics.DrawLine(LightPinkPen, X, 0, X, picHeight);
			}

			/// Drawing the horizontal lines
			for(double Counter = 0 ; Counter < Environment.GroundHeight ; Counter = Counter + Environment.CellsWidth)
			{
				int Y = (int)(picHeight - PicPixPerMeter * Counter);
				e.Graphics.DrawLine(LightPinkPen, 0, Y, picWidth, Y);
			}

			int NumOfCellsInX = (int)(Environment.GroundWidth / Environment.CellsWidth);
			int NumOfCellsInY = (int)(Environment.GroundHeight / Environment.CellsWidth);
			for(int i = 0 ; i < NumOfCellsInX ; i++)
				for(int j = 0 ; j < NumOfCellsInY ; j++)
				{
					int CellTopLeftX = (int)(i * Environment.CellsWidth * PicPixPerMeter);
					int CellTopLeftY = picHeight - (int)((j+1) * Environment.CellsWidth * PicPixPerMeter);
					if (Environment.Cells[i,j] > 0.0)
						e.Graphics.FillRectangle(
							LightPinkBrush,
							CellTopLeftX,
							CellTopLeftY,
							(float)(Environment.CellsWidth * PicPixPerMeter),
							(float)(Environment.CellsWidth * PicPixPerMeter));

				}

//			for (int i=0 ; i<Environment.GridWidth ; i++)
//				for (int j=0 ; j<Environment.GridHeight ; j++)
//				{
//					textX = picGroundWidth * (i) / Environment.GridWidth;
//					textY = picGroundHeight * (Environment.GridHeight-j-1) / Environment.GridHeight;
//					e.Graphics.DrawString(Environment.Stress[i,j].ToString("0.00"), drawFont, drawBrush, textX, textY);
//				}
		}

		/// <summary>
		/// Draws grids in the environment
		/// </summary>
		/// <param name="e"></param>
		private void DrawGrids(System.Windows.Forms.PaintEventArgs e)
		{
			/// Drawing the vertical lines
			for(double Counter=GridsDistance ; Counter < Environment.GroundWidth ; Counter = Counter + GridsDistance)
			{
				int X = (int)(PicPixPerMeter * Counter);
				e.Graphics.DrawLine(LightBluePen, X, 0, X, picHeight);
			}

			/// Drawing the horizontal lines
			for(double Counter=GridsDistance ; Counter < Environment.GroundHeight ; Counter = Counter + GridsDistance)
			{
				int Y = (int)(picHeight - PicPixPerMeter * Counter);
				e.Graphics.DrawLine(LightBluePen, 0, Y, picWidth, Y);
			}
		}

		private void DrawRobotSpring(MobileRobot robot, System.Windows.Forms.PaintEventArgs e)
		{
			foreach(MobileRobot otherRob in robot.VisibleRobots)
			{
				e.Graphics.DrawLine(
					DarkBluePen,
					(float)(robot.GetRealX() * PicPixPerMeter),
					(float)(picHeight - (robot.GetRealY() * PicPixPerMeter)),
					(float)(otherRob.GetRealX() * PicPixPerMeter),
					(float)(picHeight - (otherRob.GetRealY() * PicPixPerMeter)));
			}
		}

		/// <summary>
		/// Draws a robot on the screen
		/// </summary>
		/// <param name="robot">The robot to be drawn</param>
		private void DrawRobot(MobileRobot robot, System.Windows.Forms.PaintEventArgs e)
		{
			/// Robots position according to what (him/her)self thinks
			e.Graphics.DrawEllipse(
				RedPen,
				(float)((robot.ImX - robot.Radius) * PicPixPerMeter),
				(float)(picHeight - (robot.ImY + robot.Radius) * PicPixPerMeter),
				(float)(2 * robot.Radius * PicPixPerMeter),
				(float)(2 * robot.Radius * PicPixPerMeter));

			/*
			/// Real Robots position in the environment
			e.Graphics.DrawEllipse(
				DarkBluePen,
				(float)((robot.GetRealX() - robot.Radius) * PicPixPerMeter),
				(float)(picHeight - (robot.GetRealY() + robot.Radius) * PicPixPerMeter),
				(float)(2 * robot.Radius * PicPixPerMeter),
				(float)(2 * robot.Radius * PicPixPerMeter));
*/
			e.Graphics.DrawRectangle(
				DarkBluePen,
				(float)((robot.GetRealX() - robot.Radius) * PicPixPerMeter),
				(float)(picHeight - (robot.GetRealY() + robot.Radius) * PicPixPerMeter),
				(float)(2 * robot.Radius * PicPixPerMeter),
				(float)(2 * robot.Radius * PicPixPerMeter));

			e.Graphics.DrawEllipse(
				LightGreenPen,
				(float)((robot.GetRealX() - robot.SeeDomain) * PicPixPerMeter),
				(float)(picHeight - (robot.GetRealY() + robot.SeeDomain) * PicPixPerMeter),
				(float)(2 * robot.SeeDomain * PicPixPerMeter),
				(float)(2 * robot.SeeDomain * PicPixPerMeter));
		}

		/// <summary>
		/// Draws an obstacle on the screen
		/// </summary>
		/// <param name="obstacle">The obstacle to be drawn</param>
		private void DrawObstacle(Obstacle obstacle, System.Windows.Forms.PaintEventArgs e)
		{
			e.Graphics.FillEllipse(
				DarkRedBrush,
				(float)((obstacle.X - obstacle.Radius) * PicPixPerMeter),
				(float)(picHeight - (obstacle.Y + obstacle.Radius) * PicPixPerMeter),
				(float)(2 * obstacle.Radius * PicPixPerMeter),
				(float)(2 * obstacle.Radius * PicPixPerMeter));
		}

		/// <summary>
		/// Draws a wall on the screen
		/// </summary>
		/// <param name="obstacle">The wall to be drawn</param>
		private void DrawWall(Wall wall, System.Windows.Forms.PaintEventArgs e)
		{
			e.Graphics.FillRectangle(
				DarkBlueBrush,
				(float)(wall.X * PicPixPerMeter),
				(float)(picHeight - wall.Y * PicPixPerMeter),
				(float)(wall.Width * PicPixPerMeter),
				(float)(wall.Height * PicPixPerMeter));
		}

		/// <summary>
		/// Draws a goal on the screen
		/// </summary>
		/// <param name="goal">The goal to be drawn</param>
		private void DrawGoal(Goal goal, System.Windows.Forms.PaintEventArgs e)
		{
			e.Graphics.FillEllipse(
				GreenBrush,
				(float)((goal.X - goal.Radius) * PicPixPerMeter),
				(float)(picHeight - (goal.Y + goal.Radius) * PicPixPerMeter),
				(float)(2 * goal.Radius * PicPixPerMeter),
				(float)(2 * goal.Radius * PicPixPerMeter));
		}

		private void MoveRobotsToOrigin(object sender, System.EventArgs e)
		{
			int NumOfRobots = Environment.MobileRobots.Count;
			MobileRobot tempRob;
			for (int Counter = 1 ; Counter <= NumOfRobots ; Counter++)
			{
				tempRob = (MobileRobot)Environment.MobileRobots[Counter-1];
				tempRob.MoveTo((Counter * Environment.GroundWidth) / (NumOfRobots+1), 15);
			}
		}

		private void AddRobot(object sender, System.EventArgs e)
		{
			int NumOfRobots = Convert.ToInt32(txtNumOfRobotsToAdd.Text);
			MobileRobot tempRobot;
			for (int Counter = 1 ; Counter <= NumOfRobots ; Counter++)
			{
				tempRobot = new MobileRobot(Environment, RandomNum);

				tempRobot.MoveTo((Counter * Environment.GroundWidth) / (NumOfRobots+1), 0.5);
				tempRobot.Radius = 0.16;
				tempRobot.Load = 0.2;
				tempRobot.WorkMode = RobotsWorkMode;
				if (rdbCorrcetionNone.Checked == true)
				{
					tempRobot.CorrectionMethod = RobotCorrectionMethods.None;
				}
				else
				{
					if (rdbCorrectionAverage.Checked == true)
					{
						tempRobot.CorrectionMethod = RobotCorrectionMethods.Average;
					}
					else
					{
						if (rdbCorrectionWeightedAverage.Checked == true)
						{
							tempRobot.CorrectionMethod = RobotCorrectionMethods.WightedAverage;
						}
					}
				}

				AddRobotToGroup(tempRobot);
			}

			picGround.Refresh();
		}

		private void AddRobotToGroup(MobileRobot robot)
		{
			Environment.MobileRobots.Add(robot);
		}
		
		private void AddObstacleToGroup(Obstacle obstacle)
		{
			Environment.Obstacles.Add(obstacle);
		}

		private void AddWallToGroup(Wall wall)
		{
			Environment.Walls.Add(wall);
		}
		
		private void AddGoalToGroup(Goal goal)
		{
			Environment.Goals.Add(goal);
		}

		private void SingleStep()
		{
			if (CanShowErrors)
				lblErrorsValue.Text = Environment.TotalError().ToString("0.00");

			if (CanShowCoverageRatio)
				ShowCoverageRatio();

			if (CanShowNumOfRobotMoves)
				ShowNumOfRobotMoves();

			if (CanReport)
				if (Environment.MobileRobots.Count > 0)
				{
					Environment.RepErrors.Add(Environment.TotalError());
					Environment.RepRectangleArea.Add(Environment.TotalRobotsRectangleArea());
				}

			foreach(MobileRobot robot in Environment.MobileRobots)
				robot.Decide(MainTimerInterval);

			foreach(MobileRobot robot in Environment.MobileRobots)
				robot.Do(MainTimerInterval);

			foreach(Obstacle obstacle in Environment.Obstacles)
				obstacle.DecideAndDo(MainTimerInterval);

			foreach(Wall wall in Environment.Walls)
				wall.DecideAndDo(MainTimerInterval);

			foreach(Goal goal in Environment.Goals)
				goal.DecideAndDo(MainTimerInterval);

			if (Environment.MobileRobots.Count > 0)
				this.Text = ((MobileRobot)(Environment.MobileRobots[0])).DebugText;

			picGround.Refresh();
		}
		
		private void tmrMain_Tick(object sender, System.EventArgs e)
		{
			SingleStep();
		}

		private void picGround_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			if (CanShowOnPicture)
			{
				e.Graphics.Clear(Color.FromArgb(255, 255, 255));

//				ShowImage();

				if (ShowsEnvCells)
					DrawEnvCells(e);

				if (ShowsGrids)
					DrawGrids(e);

				if (LeftMouseIsDown)
					e.Graphics.DrawEllipse(
						RedPen,
						(float)((LeftMouseX-0.5) * PicPixPerMeter),
						(float)(picHeight - (LeftMouseY+0.5)*PicPixPerMeter),
						(float)(1 * PicPixPerMeter),
						(float)(1 * PicPixPerMeter));

				foreach(MobileRobot mRobot in Environment.MobileRobots)
					DrawRobot(mRobot, e);

//				foreach(MobileRobot mRobot in Environment.MobileRobots)
//					DrawRobotSpring(mRobot, e);

//				foreach(Obstacle obs in Environment.Obstacles)
//					DrawObstacle(obs, e);

//				foreach(Wall theWall in Environment.Walls)
//					DrawWall(theWall, e);

				foreach(Goal theGoal in Environment.Goals)
					DrawGoal(theGoal, e);

//				DrawCreatures(e);
			}
		}

		private void btnAddObstacle_Click(object sender, System.EventArgs e)
		{
			Obstacle tempObstacle = new Obstacle();

			tempObstacle.X = 4;
			tempObstacle.Y = 3.5;
			tempObstacle.Radius = 0.5;
			tempObstacle.Load = 1;

			AddObstacleToGroup(tempObstacle);

			tempObstacle = new Obstacle();

			tempObstacle.X = 6;
			tempObstacle.Y = 6.5;
			tempObstacle.Radius = 0.5;
			tempObstacle.Load = 1;

			AddObstacleToGroup(tempObstacle);

			tempObstacle = new Obstacle();

			tempObstacle.X = 3;
			tempObstacle.Y = 2;
			tempObstacle.Radius = 0.5;
			tempObstacle.Load = 1;

			AddObstacleToGroup(tempObstacle);
		}

		private void btnAddGoal_Click(object sender, System.EventArgs e)
		{
			Goal tempGoal = new Goal(Environment);

			tempGoal.X = 3.7;
			tempGoal.Y = 7.6;
			tempGoal.Radius = 0.1;
			tempGoal.Load = -300.0;

			AddGoalToGroup(tempGoal);
		}

		private void picGround_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				LeftMouseIsDown = false;
			}
			else
			{
				if (e.Button == MouseButtons.Right)
					RightMouseIsDown = false;
			}
		}

		private void MoveGoals(double x, double y)
		{
			foreach(Goal goal in Environment.Goals)
			{
				goal.MoveTo(x, y);
			}
		}

		private void btnChangeObstaclesLoad_Click(object sender, System.EventArgs e)
		{
			double NewLoadValue = Convert.ToDouble(txtNewObstaclesLoad.Text);
			foreach(Obstacle obs in Environment.Obstacles)
				obs.Load = NewLoadValue;
			foreach(Wall w in Environment.Walls)
				w.Load = NewLoadValue;
		}

		private void btnTest_Click(object sender, System.EventArgs e)
		{
//			((MobileRobot)(Environment.MobileRobots[0])).Load = Convert.ToDouble(txtTest.Text);

//			this.Text = Environment.TotalError().ToString();

//			foreach(ImageAgent imagent in Environment.IPAgents)
//			{
//				for (int i=0 ; i < TestBMP.Size.Width-1 ; i++)
//					for (int j=0 ; j < TestBMP.Size.Height-1 ; j++)
//					{
//						imagent.X = i;
//						imagent.Y = j;
//
//						imagent.Decide();
//						imagent.Do();
//					}
//			}

			Image tempImage = picGround.Image;
			Graphics gr = Graphics.FromImage(tempImage);
			gr.DrawLine(RedPen, 0, 0, 50, 50);


			// Create image.
//			Image imageFile = Image.FromFile("SampImag.jpg");
			// Create graphics object for alteration.
//			Graphics newGraphics = Graphics.FromImage(imageFile);
			// Alter image.
//			newGraphics.FillRectangle(new SolidBrush(Color.Black), 100, 50, 100, 100);
			// Draw image to screen.
//			e.Graphics.DrawImage(imageFile, new PointF(0.0F, 0.0F));
			// Release graphics object.
//			newGraphics.Dispose();


		}

		private void picGround_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Right)
			{
				RightMouseIsDown = true;
//				MoveGoals(e.X / PicPixPerMeter,(picHeight - e.Y) / PicPixPerMeter);

//				this.Text = "X == " + e.X.ToString() + "      Y == " + e.Y.ToString();

				((ImageAgent)(Environment.IPAgents[0])).X = e.X;
				((ImageAgent)(Environment.IPAgents[0])).Y = e.Y;

				((ImageAgent)(Environment.IPAgents[0])).Decide();
				((ImageAgent)(Environment.IPAgents[0])).Do();

				MainBMP = ((ImageAgent)(Environment.IPAgents[0])).Image;

				ShowImage();
			}

			if (e.Button == MouseButtons.Left)
			{
				LeftMouseIsDown = true;

				LeftMouseX = e.X / PicPixPerMeter;
				LeftMouseY = (picHeight - e.Y) / PicPixPerMeter;

				if ((LeftMouseX >= 0) && (LeftMouseY >= 0))
				{
//					((MobileRobot)(Environment.MobileRobots[0])).AcX = LeftMouseX;
//					((MobileRobot)(Environment.MobileRobots[0])).AcY = LeftMouseY;
//
//					((MobileRobot)(Environment.MobileRobots[0])).ImX = LeftMouseX;
//					((MobileRobot)(Environment.MobileRobots[0])).ImY = LeftMouseY;

					if (Environment.MobileRobots.Count > 0)
						((MobileRobot)(Environment.MobileRobots[0])).MoveTo(LeftMouseX, LeftMouseY);
				}

//				ArrayList RobsInCircle = new ArrayList();
//				Environment.RobotsInCircle(
//					LeftMouseX,
//					LeftMouseY,
//					1,
//					ref RobsInCircle);
//				this.Text = RobsInCircle.Count.ToString();
			}
		}

		private void picGround_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (RightMouseIsDown)
			{
//				MoveGoals(e.X / PicPixPerMeter,(picHeight - e.Y) / PicPixPerMeter);

				((ImageAgent)(Environment.IPAgents[0])).X = e.X;
				((ImageAgent)(Environment.IPAgents[0])).Y = e.Y;

				((ImageAgent)(Environment.IPAgents[0])).Decide();
				((ImageAgent)(Environment.IPAgents[0])).Do();

				MainBMP = ((ImageAgent)(Environment.IPAgents[0])).Image;

				ShowImage();
			}
			else
			{
				if (LeftMouseIsDown)
				{
					LeftMouseX = e.X / PicPixPerMeter;
					LeftMouseY = (picHeight - e.Y) / PicPixPerMeter;

//					((MobileRobot)(Environment.MobileRobots[0])).AcX = LeftMouseX;
//					((MobileRobot)(Environment.MobileRobots[0])).AcY = LeftMouseY;
//
//					((MobileRobot)(Environment.MobileRobots[0])).ImX = LeftMouseX;
//					((MobileRobot)(Environment.MobileRobots[0])).ImY = LeftMouseY;

					if (Environment.MobileRobots.Count > 0)
						((MobileRobot)(Environment.MobileRobots[0])).MoveTo(LeftMouseX, LeftMouseY);

				}
			}
		}

		private void btnChangeRobotsSpeed_Click(object sender, System.EventArgs e)
		{
			double NewRobotsSpeed = Convert.ToDouble(txtRobotsSpeed.Text);
			foreach(MobileRobot rob in Environment.MobileRobots)
				rob.Speed = NewRobotsSpeed;
		}

		private void btnChangeGoalSpeed_Click(object sender, System.EventArgs e)
		{
			foreach (Goal goal in Environment.Goals)
			{
				goal.MoveSpeed = Convert.ToDouble(txtGoalSpeed.Text);
			}
		}

		private void chkShowsOnPicture_CheckedChanged(object sender, System.EventArgs e)
		{
			CanShowOnPicture = chkShowsOnPicture.Checked;
		}

		private void chkShowErrors_CheckedChanged(object sender, System.EventArgs e)
		{
			CanShowErrors = chkShowErrors.Checked;
		}

		private void btnMultipleSteps_Click(object sender, System.EventArgs e)
		{
			bool LastMainTimerEnVal = tmrMain.Enabled;
			tmrMain.Enabled = false;

			this.Text = "While in MultiStep process";

			int NumOfSteps = Convert.ToInt32(txtMultipleSteps.Text);
			for(int i=1 ; i<=NumOfSteps ; i++)
				SingleStep();

			this.Text = "MultiStep process finished!";

			tmrMain.Enabled = LastMainTimerEnVal;
		}

		private void chkRecordData_CheckedChanged(object sender, System.EventArgs e)
		{
			CanReport = chkRecordData.Checked;
		}

		private void btnSaveReport_Click(object sender, System.EventArgs e)
		{
			RepGen.ReportFileName = txtReportFileName.Text;
			RepGen.GenerateReport();
		}

		private void chkAutomaticWork_CheckedChanged(object sender, System.EventArgs e)
		{
			tmrMain.Enabled = chkAutomaticWork.Checked;
		}

		private void ChangeRobotsCorrectionMethos(RobotCorrectionMethods cm)
		{
			foreach(MobileRobot rob in Environment.MobileRobots)
			{
				rob.CorrectionMethod = cm;
			}
		}

		private void ChangeRobotsWorkingMode(RobotWorkingModes rwm)
		{
			foreach(MobileRobot rob in Environment.MobileRobots)
			{
				rob.WorkMode = rwm;
			}
		}

		private void RadoiRobsWorkModeChanged(object sender, System.EventArgs e)
		{
			if (rdbRobWorkModeCompCoverage.Checked)
			{
				RobotsWorkMode = RobotWorkingModes.CompleteCoverage;
			}
			else
			{
				if (rdbRobWorkModeFollowTarget.Checked)
				{
					RobotsWorkMode = RobotWorkingModes.TargetFollowing;
				}
			}

			ChangeRobotsWorkingMode(RobotsWorkMode);
		}

		private void RadioCorrectionMethodChanged(object sender, System.EventArgs e)
		{
			if (rdbCorrcetionNone.Checked)
			{
				ChangeRobotsCorrectionMethos(RobotCorrectionMethods.None);
			}
			else
			{
				if (rdbCorrectionAverage.Checked)
				{
					ChangeRobotsCorrectionMethos(RobotCorrectionMethods.Average);
				}
				else
				{
					if (rdbCorrectionWeightedAverage.Checked)
					{
						ChangeRobotsCorrectionMethos(RobotCorrectionMethods.WightedAverage);
					}
					else
					{
						if (rdbCorrectionOWA.Checked)
						{
							ChangeRobotsCorrectionMethos(RobotCorrectionMethods.OWA);
						}
					}
				}
			}
		}

		private void btnCleanUpEntirely_Click(object sender, System.EventArgs e)
		{
			Environment.CleanUpEntirely();
		}

		private void ShowEnvCellsCheckedChanged(object sender, System.EventArgs e)
		{
			if (IsChangingShowEnvCells)
			{
			}
			else
			{
				IsChangingShowEnvCells = true;

				ShowsEnvCells = !ShowsEnvCells;

				chkShowEnvCells.Checked = ShowsEnvCells;
				menViewCells.Checked = ShowsEnvCells;

				IsChangingShowEnvCells = false;
			}
		}

		private void ShowEnvGridsCheckedChanged(object sender, System.EventArgs e)
		{
			if (IsChangingShowEnvGrids)
			{
			}
			else
			{
				IsChangingShowEnvGrids = true;

				ShowsGrids = !ShowsGrids;

				chkShowGrids.Checked = ShowsGrids;
				menViewGrids.Checked = ShowsGrids;

				IsChangingShowEnvGrids = false;
			}
		}

		private void DrawCreatures(System.Windows.Forms.PaintEventArgs e)
		{
			for (int i=1 ; i < 49 ; i++)
				for (int j=1 ; j < 49 ; j++)
				{
					if (Environment.Creatures[i,j] > 0.0)
					{
						e.Graphics.FillRectangle(
							GreenBrush,
							(float)(i * PicPixPerMeter),
							(float)(picHeight - (j + 1) * PicPixPerMeter),
							(float)(1 * PicPixPerMeter),
							(float)(1 * PicPixPerMeter));
					}
				}
		}

		private void OpenEnvironment(object sender, System.EventArgs e)
		{
			OpenFileDialog OpFD = new OpenFileDialog();
			if (OpFD.ShowDialog() == DialogResult.OK)
			{
				Environment.Walls.Clear();
				
				StreamReader TR = new StreamReader(OpFD.FileName);
				while (true)
				{
					string str = TR.ReadLine();
					if(str == null) break;
					char[] seps = new char[] {' '};
					string[] sArray = str.Split(seps);

					Wall tempWall = new Wall();

					int WallX = int.Parse(sArray[0]);
					int WallY = int.Parse(sArray[1]) + 1;

					tempWall.X = WallX;
					tempWall.Y = WallY;
					tempWall.Width = 1.0;
					tempWall.Height = 1.0;
					tempWall.Load = 4.0;

					Environment.Walls.Add(tempWall);

					Environment.Cells[WallX, WallY-1] = 1.0;
				}
			}
		}

		private void btnApplyRobotSettings_Click(object sender, System.EventArgs e)
		{
			double SpLengthVal = Convert.ToDouble(txtSpringLength.Text);
			double RobSeeRadius = Convert.ToDouble(txtRobotSeeDomain.Text);
			double RobLoadVal = Convert.ToDouble(txtRobotLoad.Text);

			foreach (MobileRobot rob in Environment.MobileRobots)
			{
				rob.SpringLength = SpLengthVal;
				rob.SeeDomain = RobSeeRadius;
				rob.Load = RobLoadVal;
			}
		}

		private void btnClearCells_Click(object sender, System.EventArgs e)
		{
			for (int i=0 ; i < 50 ; i++)
				for (int j=0 ; j < 50 ; j++)
					Environment.Cells[i,j] = 0.0;

			foreach(MobileRobot rob in Environment.MobileRobots)
			{
				rob.NumOfCellChanges = 0;
			}
		}

		private void ShowNumOfRobotMoves()
		{
			int Sum = 0;
			foreach(MobileRobot rob in Environment.MobileRobots)
			{
				Sum = Sum + rob.NumOfCellChanges;
			}

			lblNumOfRobotMoves.Text = Sum.ToString() + " total movements for all robots";
		}

		private void ShowCoverageRatio()
		{
			int NumOfCoveredCells = 0;
			int CellsTotalCoveredNum = 0;
			double CovRatio = 0;

			for (int i=0 ; i < 50 ; i++)
				for (int j=0 ; j < 50 ; j++)
				{
					if (Environment.Cells[i,j] > 0.0)
						NumOfCoveredCells++;

					CellsTotalCoveredNum = CellsTotalCoveredNum + (int)Environment.Cells[i,j];
				}

			double AveragePerCell = 0.0;
			if (NumOfCoveredCells != 0)
				AveragePerCell = (double)CellsTotalCoveredNum/(double)NumOfCoveredCells;

			CovRatio = NumOfCoveredCells / (Environment.GroundWidth * Environment.GroundHeight);
			lblCoverageRatio.Text = 
				NumOfCoveredCells.ToString() +
				"(=%" +
				(CovRatio*100).ToString("0.0") +
				") cells covered " +
				CellsTotalCoveredNum.ToString() +
				" times, Average/Cell = " +
				AveragePerCell.ToString("0.00");
		}

		private void chkShowCoverageRatio_CheckedChanged(object sender, System.EventArgs e)
		{
			CanShowCoverageRatio = chkShowCoverageRatio.Checked;
		}

		private void chkShowNumOfRobotMoves_Click(object sender, System.EventArgs e)
		{
			CanShowNumOfRobotMoves = chkShowNumOfRobotMoves.Checked;
		}

		private void MakeFirstRobotLeader(object sender, System.EventArgs e)
		{
			((MobileRobot)(Environment.MobileRobots[0])).WorkMode = RobotWorkingModes.Leader;
			((MobileRobot)(Environment.MobileRobots[0])).Load = 3.5;
		}

		private void btnCreateCreatures_Click(object sender, System.EventArgs e)
		{
			Environment.Creatures[14,14] = 1.0;
			Environment.Creatures[14,15] = 1.0;
			Environment.Creatures[14,16] = 1.0;

			Environment.Creatures[15,14] = 1.0;
			Environment.Creatures[15,15] = 1.0;
			Environment.Creatures[15,16] = 1.0;

			Environment.Creatures[16,14] = 1.0;
			Environment.Creatures[16,15] = 1.0;
			Environment.Creatures[16,16] = 1.0;

			for (int i=0 ; i < 50 ; i++)
				for (int j=0 ; j < 50 ; j++)
				{
					Environment.NextCreatures[i,j] = Environment.Creatures[i,j];
				}
		}

		private void btnCreaturesSingleStep_Click(object sender, System.EventArgs e)
		{
			Environment.DecideForCreatures();
			Environment.DoForCreatures();

			picGround.Refresh();
		}

		private void ShowImage()
		{
			TempBMP = new Bitmap(MainBMP);

			picGround.Image = TempBMP;

			Image tempImage = /*picGround.Image;*/ TempBMP;
			Graphics gr = Graphics.FromImage(tempImage);

			foreach(ImageAgent imagent in Environment.IPAgents)
			{
//				gr.DrawLine(RedPen, imagent.X, imagent.Y, imagent.X+1, imagent.Y+1);
				gr.FillRectangle(DarkRedBrush, imagent.X-1, imagent.Y-1, 2, 2);
			}

			
//			picGround.Image = TempBMP;

//			picGround.Image = TempBMP;
//			picGround.Refresh();
		}

		private void btnTestImage_Click(object sender, System.EventArgs e)
		{
/*			Color PixColor;

			for (int i=0 ; i < Environment.Image.Size.Width ; i++)
				for (int j=0 ; j < Environment.Image.Size.Height ; j++)
				{
					PixColor = Environment.Image.GetPixel(i, j);
					Environment.Image.SetPixel(i, j, Color.FromArgb((int)(255*PixColor.GetBrightness()), (int)(255*PixColor.GetBrightness()), (int)(255*PixColor.GetBrightness())));
				}

			ShowImage();*/

			Environment.MakePicBlackWhite();

			MainBMP = ((ImageAgent)(Environment.IPAgents[0])).Image;

			ShowImage();


/*
			foreach(ImageAgent ipagent in Environment.IPAgents)
				ipagent.Decide();

			foreach(ImageAgent ipagent in Environment.IPAgents)
				ipagent.Do();
*/
/*
			Color PixColor, RightPixColor, BottomPixColor;

			for (int i=0 ; i < TestBMP.Size.Width-1 ; i++)
				for (int j=0 ; j < TestBMP.Size.Height-1 ; j++)
				{
					PixColor = TestBMP.GetPixel(i, j);
					RightPixColor = TestBMP.GetPixel(i+1, j);
					BottomPixColor = TestBMP.GetPixel(i, j+1);

					double NewVal = Math.Sqrt(
						Math.Pow(PixColor.GetBrightness() - RightPixColor.GetBrightness(), 2)
						+
						Math.Pow(PixColor.GetBrightness() - BottomPixColor.GetBrightness(), 2)
						);

					PixColor = Color.FromArgb((int)(NewVal*255), (int)(NewVal*255), (int)(NewVal*255));
					
					TestBMP.SetPixel(i, j, PixColor);
				}

			ShowImage();*/

			TempBMP = new Bitmap(MainBMP);
			picGround.Image = TempBMP;
			ShowImage();
		}

		private void btnShowImage_Click(object sender, System.EventArgs e)
		{
			chkShowsOnPicture.Checked = false;
			picGround.Image = TempBMP;
			ShowImage();
		}

		private void btnImageSingleStep_Click(object sender, System.EventArgs e)
		{
			foreach(ImageAgent ipagent in Environment.IPAgents)
				ipagent.Decide();

			foreach(ImageAgent ipagent in Environment.IPAgents)
				ipagent.Do();

			MainBMP = ((ImageAgent)(Environment.IPAgents[0])).Image;

			ShowImage();
		}
	}
}
